# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
flow create_property_setter_template_model__using_parsed_fy_py_file -> PropertySetterTemplateModel:
    property parsed_fy_py_file using setter
fy"""

from typing import Any

from fy_core.base.flow_base import FlowBase
from fy_library.domain.fy_py_template_models import PropertySetterTemplateModel
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile, PropertySetterFyPyFile
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.using_setter import (
    ParsedFyPyFile_UsingSetter_PropertyMixin,
)


# fy:start ===>>>
class CreatePropertySetterTemplateModel_UsingParsedFyPyFile_Flow(
    # Property Mixins
    ParsedFyPyFile_UsingSetter_PropertyMixin,
    # Base
    FlowBase[PropertySetterTemplateModel],
):
    def __init__(
        self,
        *args: Any,
        parsed_fy_py_file: ParsedFyPyFile,
        **kwargs: Any,
    ):
        self._parsed_fy_py_file = parsed_fy_py_file
        super().__init__(*args, **kwargs)

    def __call__(self) -> PropertySetterTemplateModel:
        # fy:end <<<===
        parsed_property_setter_fy_py_file = self._parsed_fy_py_file
        assert isinstance(parsed_property_setter_fy_py_file, PropertySetterFyPyFile)
        return PropertySetterTemplateModel(
            python_class_name=parsed_property_setter_fy_py_file.python_class_name,
            property_name=parsed_property_setter_fy_py_file.property_name,
            property_type=parsed_property_setter_fy_py_file.property_type,
            generics_def=parsed_property_setter_fy_py_file.generics_def,
        )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.mixins.property.parsed_file_split.abstract_method_file_split.abc_fy import AbstractMethodFileSplitModel


property abstract_method_file_split: AbstractMethodFileSplitModel using abstract_method_regex:
    property fy_py_file_to_parse
    property fy_code
fy"""

import abc
import re
from functools import cached_property
from typing import Final

from fy_library.constants import (
    FY_ENTITY_REGEX_STRING,
    PYTHON_ARGUMENTS_REGEX_STRING,
    PYTHON_MULTI_ENTITY_REGEX_STRING,
)
from fy_library.mixins.property.fy_file.fy_code.abc_fy import FyCode_PropertyMixin_ABC
from fy_library.mixins.property.fy_py_file.fy_py_file_to_parse.abc_fy import (
    FyPyFileToParse_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_file_split.abstract_method_file_split.abc_fy import (
    AbstractMethodFileSplit_PropertyMixin_ABC,
    AbstractMethodFileSplitModel,
)

_ABSTRACT_METHOD_REGEX: Final = re.compile(
    rf"method\s+(?P<abstract_method_name>{FY_ENTITY_REGEX_STRING})"
    rf"\s*(?:\[(?P<generics>{PYTHON_MULTI_ENTITY_REGEX_STRING})])?"
    rf"\s*(?:\((?P<arguments>{PYTHON_ARGUMENTS_REGEX_STRING})\))?"
    rf"\s*(?:->\s*(?P<return_type>{PYTHON_MULTI_ENTITY_REGEX_STRING}))?\s*$",
)


# fy:start ===>>>
class AbstractMethodFileSplit_UsingAbstractMethodRegex_PropertyMixin(
    # Property Mixins
    AbstractMethodFileSplit_PropertyMixin_ABC,
    FyCode_PropertyMixin_ABC,
    FyPyFileToParse_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _abstract_method_file_split(self) -> AbstractMethodFileSplitModel:
        # fy:end <<<===
        abstract_method_file_split = _ABSTRACT_METHOD_REGEX.split(self._fy_code)

        assert (
            len(abstract_method_file_split) == 6
        ), f"Abstract Method file split length {len(abstract_method_file_split)} is invalid in {self._fy_py_file_to_parse}."

        assert (
            abstract_method_file_split[4] is not None
        ), f"Abstract method requires return type in {self._fy_py_file_to_parse}."

        abstract_method_file_split_model = AbstractMethodFileSplitModel(
            user_imports=abstract_method_file_split[0],
            abstract_method_name=abstract_method_file_split[1],
            generics_def=abstract_method_file_split[2] or "",
            arguments=abstract_method_file_split[3],
            return_type=abstract_method_file_split[4] or "",
        )

        return abstract_method_file_split_model



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.mixin_models import AbstractPropertyModel


property optional_abstract_property_mixin_model: AbstractPropertyModel | None
fy"""

from fy_library.domain.mixin_models import AbstractPropertyModel
import abc


# fy:start ===>>>
class OptionalAbstractPropertyMixinModel_PropertyMixin_ABC(abc.ABC):
    @property
    @abc.abstractmethod
    def _optional_abstract_property_mixin_model(self) -> AbstractPropertyModel | None:
        raise NotImplementedError()
        # fy:end <<<===
